@@ -38,6 +38,7 @@
 #include "lib/hci.h"
 #include "lib/l2cap.h"
 #include "lib/a2mp.h"
+#include "lib/amp.h"
 
 typedef struct {
 	uint16_t handle;
@@ -971,7 +972,8 @@ static inline void create_req(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 	if (p_filter(FILT_L2CAP))
 		return;
 
-	printf("Create req: psm %d scid 0x%4.4x id %d\n", psm, scid, h->id);
+	printf("Create chan req: psm 0x%4.4x scid 0x%4.4x ctrl id %d\n",
+							psm, scid, h->id);
 }
 
 static inline void create_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
@@ -985,7 +987,8 @@ static inline void create_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 	if (p_filter(FILT_L2CAP))
 		return;
 
-	printf("Create rsp: dcid 0x%4.4x scid 0x%4.4x result %d status %d\n", dcid, scid, result, status);
+	printf("Create chan rsp: dcid 0x%4.4x scid 0x%4.4x result %d status %d\n",
+						dcid, scid, result, status);
 }
 
 static inline void move_req(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
@@ -996,7 +999,7 @@ static inline void move_req(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 	if (p_filter(FILT_L2CAP))
 		return;
 
-	printf("Move req: icid 0x%4.4x id %d\n", icid, h->id);
+	printf("Move chan req: icid 0x%4.4x ctrl id %d\n", icid, h->id);
 }
 
 static inline void move_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
@@ -1008,7 +1011,7 @@ static inline void move_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 	if (p_filter(FILT_L2CAP))
 		return;
 
-	printf("Move rsp: icid 0x%4.4x result %d\n", icid, result);
+	printf("Move chan rsp: icid 0x%4.4x result %d\n", icid, result);
 }
 
 static inline void move_cfm(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
@@ -1020,7 +1023,7 @@ static inline void move_cfm(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 	if (p_filter(FILT_L2CAP))
 		return;
 
-	printf("Move cfm: icid 0x%4.4x result %d\n", icid, result);
+	printf("Move chan cfm: icid 0x%4.4x result %d\n", icid, result);
 }
 
 static inline void move_cfm_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
@@ -1031,7 +1034,7 @@ static inline void move_cfm_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm
 	if (p_filter(FILT_L2CAP))
 		return;
 
-	printf("Move cfm rsp: icid 0x%4.4x\n", icid);
+	printf("Move chan cfm rsp: icid 0x%4.4x\n", icid);
 }
 
 static inline void a2mp_command_rej(int level, struct frame *frm)
@@ -1162,29 +1165,13 @@ static inline void a2mp_assoc_req(int level, struct frame *frm)
 	printf("Get AMP Assoc req: id %d\n", h->id);
 }
 
-static inline void a2mp_assoc_dump(int level, uint8_t *assoc, uint16_t len)
-{
-	int i;
-
-	p_indent(level, 0);
-	printf("Assoc data:");
-	for (i = 0; i < len; i++) {
-		if (!(i%16)) {
-			printf("\n");
-			p_indent(level+1, 0);
-		}
-		printf("%2.2x ",*assoc++);
-	}
-	printf("\n");
-}
-
 static inline void a2mp_assoc_rsp(int level, struct frame *frm, uint16_t len)
 {
 	struct a2mp_assoc_rsp *h = frm->ptr;
 
-	printf("Get AMP Assoc rsp: id %d status (%d) %s \n",
-		   h->id, h->status, a2mpstatus2str(h->status));
-	a2mp_assoc_dump(level + 1, (uint8_t *) &h->assoc_data, len - 2);
+	printf("Get AMP Assoc rsp: id %d status (%d) %s\n",
+			h->id, h->status, a2mpstatus2str(h->status));
+	amp_assoc_dump(level + 1, h->assoc_data, len - sizeof(*h));
 }
 
 static inline void a2mp_create_req(int level, struct frame *frm, uint16_t len)
@@ -1193,7 +1180,7 @@ static inline void a2mp_create_req(int level, struct frame *frm, uint16_t len)
 
 	printf("Create Physical Link req: local id %d remote id %d\n",
 		   h->local_id, h->remote_id);
-	a2mp_assoc_dump(level + 1, (uint8_t *) &h->assoc_data, len - 2);
+	amp_assoc_dump(level + 1, h->assoc_data, len - sizeof(*h));
 }
 
 static inline void a2mp_create_rsp(int level, struct frame *frm)
@@ -1502,8 +1489,9 @@ static void l2cap_parse(int level, struct frame *frm)
 			break;
 
 		case 0x17:
+		case 0x1B:
 			if (!p_filter(FILT_AVCTP))
-				avctp_dump(level, frm);
+				avctp_dump(level, frm, psm);
 			else
 				raw_dump(level + 1, frm);
 			break;
